# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.consent import Consent
from ..types.health_insurance_create_request import HealthInsuranceCreateRequest
from ..types.patient_address_with_validation import PatientAddressWithValidation
from ..types.patient_details_with_validation import PatientDetailsWithValidation
from ..types.physician_create_request_base import PhysicianCreateRequestBase
from ..types.post_order_response import PostOrderResponse
from ..types.shipping_address_with_validation import ShippingAddressWithValidation
from .raw_client import AsyncRawTestkitClient, RawTestkitClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TestkitClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTestkitClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTestkitClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTestkitClient
        """
        return self._raw_client

    def register(
        self,
        *,
        sample_id: str,
        patient_details: PatientDetailsWithValidation,
        patient_address: PatientAddressWithValidation,
        user_id: typing.Optional[str] = OMIT,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        health_insurance: typing.Optional[HealthInsuranceCreateRequest] = OMIT,
        consents: typing.Optional[typing.Sequence[Consent]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Parameters
        ----------
        sample_id : str

        patient_details : PatientDetailsWithValidation

        patient_address : PatientAddressWithValidation

        user_id : typing.Optional[str]
            The user ID of the patient.

        physician : typing.Optional[PhysicianCreateRequestBase]

        health_insurance : typing.Optional[HealthInsuranceCreateRequest]

        consents : typing.Optional[typing.Sequence[Consent]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import PatientDetailsWithValidation
        from vital import Gender
        from vital import PatientAddressWithValidation
        client = Vital(api_key="YOUR_API_KEY", )
        client.testkit.register(sample_id='sample_id', patient_details=PatientDetailsWithValidation(first_name='first_name', last_name='last_name', dob='dob', gender=Gender.FEMALE, phone_number='phone_number', email='email', ), patient_address=PatientAddressWithValidation(first_line='first_line', city='city', state='state', zip='zip', country='country', ), )
        """
        _response = self._raw_client.register(
            sample_id=sample_id,
            patient_details=patient_details,
            patient_address=patient_address,
            user_id=user_id,
            physician=physician,
            health_insurance=health_insurance,
            consents=consents,
            request_options=request_options,
        )
        return _response.data

    def create_order(
        self,
        *,
        user_id: str,
        lab_test_id: str,
        shipping_details: ShippingAddressWithValidation,
        passthrough: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters
        ----------
        user_id : str

        lab_test_id : str

        shipping_details : ShippingAddressWithValidation

        passthrough : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import ShippingAddressWithValidation
        client = Vital(api_key="YOUR_API_KEY", )
        client.testkit.create_order(user_id='user_id', lab_test_id='lab_test_id', shipping_details=ShippingAddressWithValidation(receiver_name='receiver_name', first_line='first_line', city='city', state='state', zip='zip', country='country', phone_number='phone_number', ), )
        """
        _response = self._raw_client.create_order(
            user_id=user_id,
            lab_test_id=lab_test_id,
            shipping_details=shipping_details,
            passthrough=passthrough,
            request_options=request_options,
        )
        return _response.data


class AsyncTestkitClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTestkitClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTestkitClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTestkitClient
        """
        return self._raw_client

    async def register(
        self,
        *,
        sample_id: str,
        patient_details: PatientDetailsWithValidation,
        patient_address: PatientAddressWithValidation,
        user_id: typing.Optional[str] = OMIT,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        health_insurance: typing.Optional[HealthInsuranceCreateRequest] = OMIT,
        consents: typing.Optional[typing.Sequence[Consent]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Parameters
        ----------
        sample_id : str

        patient_details : PatientDetailsWithValidation

        patient_address : PatientAddressWithValidation

        user_id : typing.Optional[str]
            The user ID of the patient.

        physician : typing.Optional[PhysicianCreateRequestBase]

        health_insurance : typing.Optional[HealthInsuranceCreateRequest]

        consents : typing.Optional[typing.Sequence[Consent]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import PatientDetailsWithValidation
        from vital import Gender
        from vital import PatientAddressWithValidation
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.testkit.register(sample_id='sample_id', patient_details=PatientDetailsWithValidation(first_name='first_name', last_name='last_name', dob='dob', gender=Gender.FEMALE, phone_number='phone_number', email='email', ), patient_address=PatientAddressWithValidation(first_line='first_line', city='city', state='state', zip='zip', country='country', ), )
        asyncio.run(main())
        """
        _response = await self._raw_client.register(
            sample_id=sample_id,
            patient_details=patient_details,
            patient_address=patient_address,
            user_id=user_id,
            physician=physician,
            health_insurance=health_insurance,
            consents=consents,
            request_options=request_options,
        )
        return _response.data

    async def create_order(
        self,
        *,
        user_id: str,
        lab_test_id: str,
        shipping_details: ShippingAddressWithValidation,
        passthrough: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters
        ----------
        user_id : str

        lab_test_id : str

        shipping_details : ShippingAddressWithValidation

        passthrough : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import ShippingAddressWithValidation
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.testkit.create_order(user_id='user_id', lab_test_id='lab_test_id', shipping_details=ShippingAddressWithValidation(receiver_name='receiver_name', first_line='first_line', city='city', state='state', zip='zip', country='country', phone_number='phone_number', ), )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_order(
            user_id=user_id,
            lab_test_id=lab_test_id,
            shipping_details=shipping_details,
            passthrough=passthrough,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .failure_type import FailureType
from .parent_biomarker_data import ParentBiomarkerData


class MissingBiomarkerResult(UniversalBaseModel):
    name: str
    slug: str
    inferred_failure_type: FailureType
    note: typing.Optional[str] = None
    loinc: typing.Optional[str] = None
    loinc_slug: typing.Optional[str] = None
    provider_id: typing.Optional[str] = None
    source_markers: typing.Optional[typing.List[ParentBiomarkerData]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Micros(UniversalBaseModel):
    minerals: typing.Optional[typing.Dict[str, typing.Optional[float]]] = pydantic.Field(default=None)
    """
    Amount of each mineral in their respective units. Most minerals are measured in mg (milligrams), while sodium and potassium are in g (grams).
    """

    trace_elements: typing.Optional[typing.Dict[str, typing.Optional[float]]] = pydantic.Field(default=None)
    """
    Amount of each trace element in their respective units, most measured in μg (micrograms), while copper and manganese are in mg (milligrams).
    """

    vitamins: typing.Optional[typing.Dict[str, typing.Optional[float]]] = pydantic.Field(default=None)
    """
    Amount of each vitamin in their respective units. A, B12, D and K in μg (micrograms), while B1, B2, B3, B5, B6, C and folic acid are in mg (milligrams).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_body_temperature_sample import ClientFacingBodyTemperatureSample
from .client_facing_source import ClientFacingSource


class GroupedBodyTemperature(UniversalBaseModel):
    source: ClientFacingSource
    data: typing.List[ClientFacingBodyTemperatureSample]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .basal_body_temperature_entry import BasalBodyTemperatureEntry
from .cervical_mucus_entry import CervicalMucusEntry
from .client_facing_menstrual_cycle_source_provider import ClientFacingMenstrualCycleSourceProvider
from .client_facing_menstrual_cycle_source_type import ClientFacingMenstrualCycleSourceType
from .client_facing_source import ClientFacingSource
from .contraceptive_entry import ContraceptiveEntry
from .detected_deviation_entry import DetectedDeviationEntry
from .home_pregnancy_test_entry import HomePregnancyTestEntry
from .home_progesterone_test_entry import HomeProgesteroneTestEntry
from .intermenstrual_bleeding_entry import IntermenstrualBleedingEntry
from .menstrual_flow_entry import MenstrualFlowEntry
from .ovulation_test_entry import OvulationTestEntry
from .sexual_activity_entry import SexualActivityEntry


class ClientFacingMenstrualCycle(UniversalBaseModel):
    id: str
    period_start: str
    period_end: typing.Optional[str] = None
    cycle_end: typing.Optional[str] = None
    is_predicted: typing.Optional[bool] = None
    menstrual_flow: typing.Optional[typing.List[MenstrualFlowEntry]] = None
    cervical_mucus: typing.Optional[typing.List[CervicalMucusEntry]] = None
    intermenstrual_bleeding: typing.Optional[typing.List[IntermenstrualBleedingEntry]] = None
    contraceptive: typing.Optional[typing.List[ContraceptiveEntry]] = None
    detected_deviations: typing.Optional[typing.List[DetectedDeviationEntry]] = None
    ovulation_test: typing.Optional[typing.List[OvulationTestEntry]] = None
    home_pregnancy_test: typing.Optional[typing.List[HomePregnancyTestEntry]] = None
    home_progesterone_test: typing.Optional[typing.List[HomeProgesteroneTestEntry]] = None
    sexual_activity: typing.Optional[typing.List[SexualActivityEntry]] = None
    basal_body_temperature: typing.Optional[typing.List[BasalBodyTemperatureEntry]] = None
    source_provider: ClientFacingMenstrualCycleSourceProvider
    source_type: ClientFacingMenstrualCycleSourceType = pydantic.Field()
    """
    ℹ️ This enum is non-exhaustive.
    """

    source_app_id: typing.Optional[str] = None
    source_device_id: typing.Optional[str] = None
    created_at: dt.datetime
    updated_at: dt.datetime
    user_id: str
    source: ClientFacingSource

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DelegatedFlowType(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    ORDER_AND_RESULTS_WITH_CUSTOMER_PHYSICIAN_NETWORK = "order_and_results_with_customer_physician_network"
    ORDER_WITH_VITAL_PHYSICIAN_NETWORK = "order_with_vital_physician_network"
    ORDER_AND_RESULTS_WITH_VITAL_PHYSICIAN_NETWORK = "order_and_results_with_vital_physician_network"
    _UNKNOWN = "__DELEGATEDFLOWTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DelegatedFlowType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        order_and_results_with_customer_physician_network: typing.Callable[[], T_Result],
        order_with_vital_physician_network: typing.Callable[[], T_Result],
        order_and_results_with_vital_physician_network: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DelegatedFlowType.ORDER_AND_RESULTS_WITH_CUSTOMER_PHYSICIAN_NETWORK:
            return order_and_results_with_customer_physician_network()
        if self is DelegatedFlowType.ORDER_WITH_VITAL_PHYSICIAN_NETWORK:
            return order_with_vital_physician_network()
        if self is DelegatedFlowType.ORDER_AND_RESULTS_WITH_VITAL_PHYSICIAN_NETWORK:
            return order_and_results_with_vital_physician_network()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingElectrocardiogramClassification(str, enum.Enum):
    SINUS_RHYTHM = "sinus_rhythm"
    ATRIAL_FIBRILLATION = "atrial_fibrillation"
    INCONCLUSIVE = "inconclusive"
    _UNKNOWN = "__CLIENTFACINGELECTROCARDIOGRAMCLASSIFICATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClientFacingElectrocardiogramClassification":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        sinus_rhythm: typing.Callable[[], T_Result],
        atrial_fibrillation: typing.Callable[[], T_Result],
        inconclusive: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClientFacingElectrocardiogramClassification.SINUS_RHYTHM:
            return sinus_rhythm()
        if self is ClientFacingElectrocardiogramClassification.ATRIAL_FIBRILLATION:
            return atrial_fibrillation()
        if self is ClientFacingElectrocardiogramClassification.INCONCLUSIVE:
            return inconclusive()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClientFacingProvider(UniversalBaseModel):
    """
    A vendor, a service, or a platform which Vital can connect with.
    """

    name: str = pydantic.Field()
    """
    Name of source of information
    """

    slug: str = pydantic.Field()
    """
    Slug for designated source
    """

    logo: str = pydantic.Field()
    """
    URL for source logo
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
